local M = {}

local running = true
local scenario = nil
local speed = 0

local helper = require('scenario/scenariohelper')

local function reset()
    running = false
end

local function fail(reason)
    scenarios.finish({failed = reason})
    reset()
end

local function success(speed)
    local result = {msg = 'Total damage done: $'..string.format("%0.2f", speed)..''}
    scenarios.finish(result)
    reset()
end

local function onRaceStart()
    reset()

	helper.setAiMode('aiCar1', 'manual')
    helper.setAiAggression('aiCar1', 2)
    helper.setAiTarget('aiCar1', '')
    helper.trackVehicle('aiCar1', 'aiCarA')
	
	helper.setAiMode('aiCar2', 'manual')
    helper.setAiAggression('aiCar2', 0.6)
    helper.setAiTarget('aiCar2', '')
    helper.trackVehicle('aiCar2', 'aiCarB')
	
	helper.setAiMode('aiCar3', 'manual')
    helper.setAiAggression('aiCar3', 0.8)
    helper.setAiTarget('aiCar3', '')
    helper.trackVehicle('aiCar3', 'aiCarC')

    helper.trackVehicle('scenario_player0', 'playerCar')

    running = true
end

local function onRaceTick(raceTickTime)
    if not running then
        return
    end
	
	local aiCarAData = map.objects[map.objectNames['aiCarA']]
	local aiCarBData = map.objects[map.objectNames['aiCarB']]
	local aiCarCData = map.objects[map.objectNames['aiCarC']]
    local playerCarData = map.objects[map.objectNames['playerCar']]

    if not playerCarData then
        return
    end
	
	local player = scenetree.findObject('scenario_player0')
	local playerPos = player:getPosition()
	local playerZ = playerPos.z
	
	local ai2Distance = helper.getDistanceBetweenSceneObjects('aiCar2', 'scenario_player0')
	local ai3Distance = helper.getDistanceBetweenSceneObjects('aiCar3', 'scenario_player0')
	
	if playerZ <= 28 then
		helper.setAiTarget('aiCar1', 'HollandPC2_AG')
	end
	
	if ai2Distance < 200 then
		helper.setAiTarget('aiCar2', 'LombardEB_AA')
	end
	
	if ai3Distance < 180 then
		helper.setAiTarget('aiCar3', 'Shepard_AWL')
	end
	
	if playerCarData.damage >= 30000 then
		fail("You crashed your car!")
	end

    if scenario.timer > 140 then
		fail("You lost!")
    end
end

local function onScenarioChange(sc)
    scenario = sc
end

M.onRaceStart = onRaceStart
M.onRaceTick = onRaceTick
M.onScenarioChange = onScenarioChange

return M