local M = {}

local running = true
local scenario = nil
local convo = 0
local cTime = 0

local helper = require('scenario/scenariohelper')

local function clearMessages()
	mesA = ""
	mesB = ""
	mesC = ""
	ui_message(mesA, 1, 'A')
	ui_message(mesB, 1, 'B')
	ui_message(mesC, 1, 'C')
end

local function reset()
    running = false
	convo = 0
	cTime = 0
	clearMessages()
	TorqueScript.eval('Music.stop();')
end

local function fail(reason)
    scenarios.finish({failed = reason})
    reset()
end

local function success(speed)
    local result = {msg = 'Hello? Hello?! You alright? Okay,... good. Damn, he must\'ve slipped something into the car while we weren\'t looking. We\'ll talk about this later.'}
    scenarios.finish(result)
    reset()
end

local function onRaceStart()
    reset()
	
	TorqueScript.eval('Music.stop();')

    helper.setAiMode('aiCar1', 'manual')
    helper.setAiAggression('aiCar1', 1)
    helper.setAiTarget('aiCar1', '')
    helper.trackVehicle('aiCar1', 'aiCarA')
	
	TextTone = scenetree.findObject('TextTone')
	Music = scenetree.findObject('Music')

    helper.trackVehicle('scenario_player0', 'playerCar')

    running = true
end

local function onRaceTick(raceTickTime)
    if not running then
        return
    end

    local aiCarAData = map.objects[map.objectNames['aiCarA']]
    local playerCarData = map.objects[map.objectNames['playerCar']]
	
	local playerDistance = helper.getDistanceBetweenSceneObjects('scenario_player0', 'aiCar1')
	
	local player = scenetree.findObject('scenario_player0')
	local playerPos = player:getPosition()
	local playerZ = playerPos.z
	
	cTime = cTime+raceTickTime

    if not aiCarAData or not playerCarData then
        return
    end
	
	if playerCarData.damage >= 30000 then
		fail("You didn't hit the target!")
	end
	
	if playerDistance > 100 then
		fail("You went the wrong way!")
	end
	
	if playerZ > 20 and playerDistance < 40 and convo == 0 then
		mesA = "vEh1CLeShUTD0wN.exe engaged..."
		ui_message(mesA, 1.95, 'A')
		local playerCar = helper.getVehicleByName('scenario_player0')
        playerCar:queueLuaCommand('scenario.freeze(1)')
		TorqueScript.eval('Music.play();')
		convo = 1	
		cTime = 0
	end
	
	if convo == 1 then
		if cTime > 2 then
			mesB = "NEW TEXT MESSAGE:"
			ui_message(mesB, 20, 'B')
			TorqueScript.eval('TextTone.play();')
			convo = 2
		end
	elseif convo == 2 then
		if cTime > 3 then
			mesC = "Your car will move when we tell it to."
			ui_message(mesC, 20, 'C')
			convo = 3
		end
	elseif convo == 3 then
		if cTime > 6 then
			clearMessages()
			TorqueScript.eval('TextTone.play();')
			mesA = "I don't think we've been properly introduced,"
			ui_message(mesA, 20, 'A')
			convo = 4
		end
	elseif convo == 4 then
		if cTime > 8 then
			mesB = "My name's Mark Freeland."
			ui_message(mesB, 20, 'B')
			convo = 5
		end
	elseif convo == 5 then
		if cTime > 9.5 then
			mesC = "If that name doesn't ring a bell, then we've been doing a good job keeping a low profile."
			ui_message(mesC, 20, 'C')
			convo = 6
		end
	elseif convo == 6 then
		if cTime > 13 then
			clearMessages()
			TorqueScript.eval('TextTone.play();')
			mesA = "I'm here because for the last few weeks,"
			ui_message(mesA, 20, 'A')
			convo = 7
		end
	elseif convo == 7 then
		if cTime > 14.5 then
			mesB = "YOU have been knocking my associates off the road."
			ui_message(mesB, 20, 'B')
			convo = 8
		end
	elseif convo == 8 then
		if cTime > 17.5 then
			clearMessages()
			TorqueScript.eval('TextTone.play();')
			mesC = "Now, don't worry. I'm not mad."
			ui_message(mesC, 20, 'C')
			convo = 9
		end
	elseif convo == 9 then
		if cTime > 18.5 then
			mesA = "In fact, I love your work."
			ui_message(mesA, 20, 'A')
			convo = 10
		end
	elseif convo == 10 then
		if cTime > 19.8 then
			mesB = "So chaotic."
			ui_message(mesB, 20, 'B')
			convo = 11
		end
	elseif convo == 11 then
		if cTime > 20.5 then
			mesC = "Inspired!"
			ui_message(mesC, 20, 'C')
			convo = 12
		end
	elseif convo == 12 then
		if cTime > 22 then
			mesA = "And that's what I want to talk to you about. I want you to work for me."
			ui_message(mesA, 20, 'A')
			convo = 13
		end
	elseif convo == 13 then
		if cTime > 25 then
			clearMessages()
			TorqueScript.eval('TextTone.play();')
			mesB = "Lots of fun opportunites, lots of destruction, not half-bad pay."
			ui_message(mesB, 20, 'B')
			convo = 14
		end
	elseif convo == 14 then
		if cTime > 27 then
			mesC = "You can stay in that little crash club, too. I like them, they're spirited.."
			ui_message(mesC, 20, 'C')
			convo = 15
		end
	elseif convo == 15 then
		if cTime > 30 then
			clearMessages()
			TorqueScript.eval('TextTone.play();')
			mesA = "And there's the catch, they don't seem to like me."
			ui_message(mesA, 20, 'A')
			convo = 16
		end
	elseif convo == 16 then
		if cTime > 33 then
			clearMessages()
			TorqueScript.eval('TextTone.play();')
			mesB = "Don't mention we talked,"
			ui_message(mesB, 20, 'B')
			convo = 17
		end
	elseif convo == 17 then
		if cTime > 34.5 then
			mesC = "and I'll let you in on a handful of small jobs, just to see how you do."
			ui_message(mesC, 20, 'C')
			convo = 18
		end
	elseif convo == 18 then
		if cTime > 37 then
			clearMessages()
			TorqueScript.eval('TextTone.play();')
			mesA = "Until then, though, I've got work to do."
			ui_message(mesA, 8, 'A')
			convo = 19
			helper.setAiTarget('aiCar1', 'EsplandeWB_BLR')
		end
	elseif convo == 19 then
		if cTime > 42 then
			TorqueScript.eval('TextTone.play();')
			mesB = "I'll keep in touch."
			ui_message(mesB, 8, 'B')
			convo = 20
		end
	end
	
	if convo == 20 and playerDistance > 90 then
		success(speed)
	end
end

local function onScenarioChange(sc)
    scenario = sc
end

M.onRaceStart = onRaceStart
M.onRaceTick = onRaceTick
M.onScenarioChange = onScenarioChange

return M